; SPU viewer for Sommarhack 2022
; Strips away the STe bit in the palette
; to make sure all entries are really ST.
;
; Builds with Devpac 3 (gen.ttp)
;
; 2022-04-24
; ae@dhs.nu


		output	.ttp

		section	text

start:
		move.l	4(sp),a5			;Address to basepage
		move.l	a5,d0				;Basepage address
		add.l	#129,d0				;Offset to commandline
		move.l	d0,fn
		move.l	$0c(a5),d0			;Length of text segment
		add.l	$14(a5),d0			;Length of data segment
		add.l	$1c(a5),d0			;Length of bss segment
		add.l	#$100,d0			;Length of basepage
		add.l	#$1000,d0			;Length of stackpointer
		move.l	a5,d1				;Address to basepage
		add.l	d0,d1				;End of program
		and.l	#-2,d1				;Make address even
		move.l	d1,sp				;New stackspace

		move.l	d0,-(sp)			;mshrink()
		move.l	a5,-(sp)
		move.w	d0,-(sp)
		move.w	#$4a,-(sp)
		trap	#1
		lea	12(sp),sp

		clr.w	-(sp)				;fopen()
		move.l	fn,-(sp)
		move.w	#$3d,-(sp)
		trap	#1
		addq.l	#8,sp
		move.w	d0,fh

		pea	spu				;fread()
		move.l	#51104,-(sp)
		move.w	fh,-(sp)
		move.w	#$3f,-(sp)
		trap	#1
		lea	12(sp),sp

		move.w	fh,-(sp)			;fclose()
		move.w	#$3e,-(sp)
		trap	#1
		addq.l	#4,sp

		pea	spuview(pc)			;Run lowlevel stuff
		move.w	#$26,-(sp)			;supexec()
		trap	#14
		addq.l	#6,sp

		clr.w	-(sp)				;pterm()
		trap	#1



spuview:	lea	save_screenadr(pc),a0		;Save old screen address
		move.b	$ffff8201.w,(a0)+
		move.b	$ffff8203.w,(a0)+
		move.b	$ffff820d.w,(a0)+

		movem.l	$ffff8240.w,d0-d7		;Save old palette
		movem.l	d0-d7,save_pal

		move.l	#screen+256,d0			;Align new screen address
		clr.b	d0
		move.l	d0,scraddr

		lsr.w	#8,d0				;Set new screen address
		move.l	d0,$ffff8200.w

		move.b	$ffff8260.w,save_res		;Save old resolution
		clr.b	$ffff8260.w			;Set low resolution

		move.b	$ffff820a.w,save_freq		;Save old refresh
		move.b	#2,$ffff820a.w			;Set 50 Hz

		move.b	#$12,$fffffc02.w		;Kill mouse


		;copy picture to screen
		lea	spu(pc),a0
		move.l	scraddr(pc),a1
		move.w	#32000/4-1,d7
.copy:		move.l	(a0)+,(a1)+
		dbra	d7,.copy

		;eliminate STe palette (no cheating!)
		lea	spu+32000(pc),a0
		move.l	#%11110111011101111111011101110111,d0
		move.w	#199*48/2-1,d7
.nostepal:	move.l	(a0),d1
		and.l	d0,d1
		move.l	d1,(a0)+
		dbra	d7,.nostepal


		move.w	#$2700,sr			;Stop all interrupts
		lea	save_vect_mfp(pc),a0
		move.l	$70.w,(a0)+			;Save old VBL
		move.l	$68.w,(a0)+			;Save old HBL
		move.l	$134.w,(a0)+			;Save old Timer A
		move.l	$120.w,(a0)+			;Save old Timer B
		move.l	$114.w,(a0)+			;Save old Timer C
		move.l	$110.w,(a0)+			;Save old Timer D
		move.l	$118.w,(a0)+			;Save old ACIA
		move.b	$fffffa07.w,(a0)+
		move.b	$fffffa09.w,(a0)+
		move.b	$fffffa13.w,(a0)+
		move.b	$fffffa15.w,(a0)+
		move.b	$fffffa19.w,(a0)+
		move.b	$fffffa1b.w,(a0)+
		move.b	$fffffa1f.w,(a0)+
		move.b	$fffffa21.w,(a0)+
		clr.b	$fffffa07.w			;Interrupt enable A (Timer-A & B)
		clr.b	$fffffa13.w			;Interrupt mask A (Timer-A & B)
		clr.b	$fffffa09.w			;Interrupt enable B (Timer-C & D)
		clr.b	$fffffa15.w			;Interrupt mask B (Timer-C & D)

		lea	dummy(pc),a0
		move.l	#vbl,$70.w			;Install our own VBL
		move.l	a0,$68.w			;Install our own HBL
		move.l	#timer_a,$134.w			;Install our own Timer A
		move.l	a0,$120.w			;Install our own Timer B
		move.l	a0,$114.w			;Install our own Timer C
		move.l	a0,$110.w			;Install our own Timer D
		move.l	a0,$118.w			;Install our own ACIA

		move.w	#$2300,sr			;Interrupts back on


		;---------------------------------------
.mainloop:	tst.w	vblcount			;Wait VBL
		beq.s	.mainloop
		clr.w	vblcount

		cmp.b	#$39,$fffffc02.w		;Space?
		bne.s	.mainloop
		;----------------------------------------


		move.w	#$2700,sr			;Stop all interrupts
		lea	save_vect_mfp(pc),a0
		move.l	(a0)+,$70.w			;Restore old VBL
		move.l	(a0)+,$68.w			;Restore old HBL
		move.l	(a0)+,$134.w			;Restore old Timer A
		move.l	(a0)+,$120.w			;Restore old Timer B
		move.l	(a0)+,$114.w			;Restore old Timer C
		move.l	(a0)+,$110.w			;Restore old Timer D
		move.l	(a0)+,$118.w			;Restore old ACIA
		move.b	(a0)+,$fffffa07.w
		move.b	(a0)+,$fffffa09.w
		move.b	(a0)+,$fffffa13.w
		move.b	(a0)+,$fffffa15.w
		move.b	(a0)+,$fffffa19.w
		move.b	(a0)+,$fffffa1b.w
		move.b	(a0)+,$fffffa1f.w
		move.b	(a0)+,$fffffa21.w
		move.w	#$2300,sr			;Interrupts back on

		move.b	save_res(pc),$ffff8260.w	;Restore old resolution
		move.b	save_freq(pc),$ffff820a.w	;Restore old refresh

		movem.l	save_pal(pc),d0-d7		;Restore old palette
		movem.l	d0-d7,$ffff8240.w

		lea	save_screenadr(pc),a0		;Restore old screen address
		move.b	(a0)+,$ffff8201.w
		move.b	(a0)+,$ffff8203.w
		move.b	(a0)+,$ffff820d.w

		move.b	#$8,$fffffc02.w			;Enable mouse

		rts


vbl:		movem.l	d0-a6,-(sp)

		addq.w	#1,vblcount

		move.w	#$2700,sr			;Stop all interrupts
		clr.b	$fffffa19.w			;Timer-A control (stop)
		bset	#5,$fffffa07.w			;Interrupt enable A (Timer A)
		bset	#5,$fffffa13.w			;Interrupt mask A (Timer A)
		move.b	#190,$fffffa1f.w		;Timer A Delay (data)
		move.b	#4,$fffffa19.w			;Timer A Predivider (start Timer A)
		bclr	#3,$fffffa17.w			;Automatic end of interrupt

		lea	$ffff8240.w,a0
		moveq	#0,d0
		moveq	#8-1,d7
.blackpal:	move.l	d0,(a0)+
		dbra	d7,.blackpal

		movem.l	(sp)+,d0-a6
		rte

dummy:		rte


timer_a:
		movem.l	d0-a6,-(sp)

		move.w	#$2100,sr			;Enable HBL
		stop	#$2100				;Wait for HBL
		move.w	#$2700,sr			;Stop all interrupts
		clr.b	$fffffa19.w			;Stop Timer A

		lea	$ffff8209.w,a0			;Hardsync
		moveq	#127,d1
.sync:		tst.b	(a0)
		beq.s	.sync
		move.b	(a0),d2
		sub.b	d2,d1
		lsr.l	d1,d1

		lea	spu+32000(pc),a0
		lea	$ffff8240.w,a1
		move.w	#199-1,d7

		nop
		dcb.w	22,$8080

.yloop:
		move.l	a1,a2				;1
		move.l	a1,a3				;1
		move.l	a1,a4				;1
		move.l	(a0)+,(a2)+			;5
		move.l	(a0)+,(a2)+			;5
		move.l	(a0)+,(a2)+			;5
		move.l	(a0)+,(a2)+			;5
		move.l	(a0)+,(a2)+			;5
		move.l	(a0)+,(a2)+			;5
		move.l	(a0)+,(a2)+			;5
		move.l	(a0)+,(a2)+			;5
		move.l	(a0)+,(a3)+			;5
		move.l	(a0)+,(a3)+			;5
		move.l	(a0)+,(a3)+			;5
		move.l	(a0)+,(a3)+			;5
		move.l	(a0)+,(a3)+			;5
		move.l	(a0)+,(a3)+			;5
		move.l	(a0)+,(a3)+			;5
		move.l	(a0)+,(a3)+			;5
		move.l	(a0)+,(a4)+			;5
		move.l	(a0)+,(a4)+			;5
		move.l	(a0)+,(a4)+			;5
		move.l	(a0)+,(a4)+			;5
		move.l	(a0)+,(a4)+			;5
		move.l	(a0)+,(a4)+			;5
		move.l	(a0)+,(a4)+			;5
		move.l	(a0)+,(a4)+			;5
		dcb.w	1,$8080
		dbra	d7,.yloop

		movem.l	(sp)+,d0-a6
		rte

		section	bss

fn:		ds.l	1
fh:		ds.w	1
scraddr:	ds.l	1
vblcount:	ds.w	1
save_pal:	ds.w	16
save_res:	ds.b	1
save_freq:	ds.b	1
save_screenadr:	ds.l	1
save_vect_mfp:	ds.b	28+8
spu:		ds.b	51104
screen:		ds.b	160*200+256
		even

		end
